;--------------------------------------------------------------------
; Description:
; This is the coded for the NMI (or non-maskable interrrupt).
; When the CPU hits the VBLANK it will suspend normal work and FORCE
; an exection of the NMI subroutine.  Once the NMI is complete, it will
; Resume normal operations again. 
;
; VARIABLES:
; **********************************************************************************
; 
; VAR.PPUCTRL holds the currenct value of the PPUCTRL as described by the encoding below
; BIT   Description
; 0-1	Base Nametable Address (0 = $2000, 1 = $2400, 2 = $2800, 3 = $2C00)
;  2    VRAM address increment per CPU read/write of PPUDATA (0 = adds 1, going accross; 1 = adds 32, goind down)
;  3    Sprite pattern table address for 8x8 sprites (0 = $0000, 1 = $1000)
;  4    Background pattern table address (0 = $0000, 1 = $1000)
;  5    Sprite size (0: 8x8; 1: 8x16)
;  6    PPU master/slave select (0: read backdrop from EXT pins; 1: output color on EXT pins)
;  7    Generate an NMI at the start of the VBLANK (0: off; 1: on)
;
; VAR.PPUMASK holds the currenct value of the PPUMASK as described by the encoding below
; BIT   Description
;  0	Greyscale (0: normal color, 1: produce a greyscale display)
;  1    1: Show background in leftmost 8 pixels of screen
;  2    1: Show sprites in leftmost 8 pixels of screen
;  3    1: Show Background
;  4    1: Show Sprites
;  5    1: Emphasize Red
;  6    1: Emphasize Green
;  7    1: Emphasize Blue
;
; The NMI uses a fixed set of FLAGS to regulate exection.
; 
; FLAG.NMI.Sprites.Palette
; is set to true when the color palettes for the sprites in the PPU needs to change.
; The sprite color palette is stored in the buffer BUF.Palette.Sprites which must be defined in the CPU RAM
;
; FLAG.NMI.Background.Palette
; is set to true when the color palettes for the background in the PPU needs to be change.
; The background color palette is stored in the buffer BUF.Palette.Background which must be defined in the CPU RAM  
;
; FLAG.NMI.Sprites
; is set to true when the sprites need to copied from the buffer to the PPU.
; The sprite buffer is stored at BUF.Sprites which is usally located at #$0200-02FF in CPU RAM
;
; FLAG.NMI.RenderingEnabled
; is usually set to true. The exeception occurs during scene initialization when the amount of data that
; needs to be copied to the PPU exceeds the VBLANK window (such as loading a nametable).
; The example code below describes the proper execution of the initialization code.
;**********************************************************************************
;
; Sets the NMI Flags
; LDA FALSE
; STA FLAG.NMI.RenderingEnabled
; STA FLAG.NMI.Background.Palette
; STA FLAG.NMI.Sprites.Palette
; STA FLAG.NMI.Sprites
;
; Rendering should only be enabled/disabled during VBLANK
; This causes the CPU to wait until the next VBLANK
; - BIT $2002
; BPL - 
;  
; Disable Rendering by writting to the PPUMASK Register 
; LDX #$00
; STX $2001
;
; (INSERT BODY HERE)
;
; Sets the NMI Flags
; LDA TRUE
; STA FLAG.NMI.RenderingEnabled
; STA FLAG.NMI.Background.Palette
; STA FLAG.NMI.Sprites.Palette
; STA FLAG.NMI.Sprites

; Rendering should only be enabled/disabled during VBLANK
; This causes the CPU to wait until the next VBLANK
; - BIT $2002
; BPL - 
;  
; Re-Enable Rendering by writting to the PPUMASK Register
; LDA VAR.PPUMASK
; STA $2001
;
; **********************************************************************************
;
;--------------------------------------------------------------------
; NMI
;--------------------------------------------------------------------
LBL.NMI:

  ;  Preserves the state of the A, X, Y registers
  PHA
  TXA
  PHA
  TYA
  PHA
  
  ;  Writes the sprite palettes on the PPU
  JSR LBL.NMI.LoadPalettes.Sprites
  
  ;  Writes the background palettes on the PPU  
  JSR LBL.NMI.LoadPalettes.Background
  
  ;  Copies the sprites buffer to the PPU OAM memory
  JSR LBL.NMI.UpdateSprites
  
  ; Enables Rendering
  JSR LBL.NMI.RenderingEnabled
  
  ;  Increments frame counter
  INC VAR.Frame
  
  ; Restores registers
  PLA
  TAY
  PLA
  TAX
  PLA
  
  ; Returns control to the CPU from the place where the NMI was forced.
  RTI
  
;--------------------------------------------------------------------
; Description:
; NMI Subroutine that copies Sprite Palette Buffer to PPU
;--------------------------------------------------------------------
LBL.NMI.LoadPalettes.Sprites:
  LDA FLAG.NMI.Sprites.Palette
  BEQ LBL.NMI.LoadPalette.Sprites.Exit
    LDA $2002
    LDA #$3F
    STA $2006
    LDA #$10
    STA $2006  
    LDX #$00
  LBL.NMI.LoadPalettes.Sprites.Loop:
    LDA BUF.Palette.Sprites, X
    STA $2007
    INX
    CPX #$10
    BNE LBL.NMI.LoadPalettes.Sprites.Loop
	
    LDA FALSE
	STA FLAG.NMI.Sprites.Palette
	
LBL.NMI.LoadPalette.Sprites.Exit:
RTS

;--------------------------------------------------------------------
; Description:
; NMI Subroutine that copies Background Palette Buffer to PPU
;--------------------------------------------------------------------
LBL.NMI.LoadPalettes.Background:
  LDA FLAG.NMI.Background.Palette
  BEQ LBL.NMI.LoadPalette.Background.Exit
    LDA $2002
    LDA #$3F
    STA $2006
    LDA #$00
    STA $2006
    LDX #$00
    LBL.NMI.LoadPalettes.Background.Loop:
      LDA BUF.Palette.Background, X
      STA $2007
      INX
      CPX #$10
      BNE LBL.NMI.LoadPalettes.Background.Loop
	  
	LDA FALSE
	STA FLAG.NMI.Background.Palette

LBL.NMI.LoadPalette.Background.Exit:
RTS

;--------------------------------------------------------------------
; Description:
; NMI Subroutine that copies Sprite Buffer to PPU
;--------------------------------------------------------------------
LBL.NMI.UpdateSprites:
  LDA FLAG.NMI.Sprites
  BEQ LBL.NMI.UpdateSprites.Exit
    LDA #$00
    STA $2003
    LDA #>BUF.Sprites
    STA $4014
    LDA FALSE
    STA FLAG.NMI.Sprites
LBL.NMI.UpdateSprites.Exit:
RTS  

;--------------------------------------------------------------------
; Description:
; NMI Subroutine that renables rendering
;--------------------------------------------------------------------
LBL.NMI.RenderingEnabled:
  LDA FLAG.NMI.RenderingEnabled
  BEQ LBL.NMI.RenderingEnabled.Exit
  
    ; Enable Rendering Register
    LDA VAR.PPUCTRL
    STA $2000
	
	; Reset Scrolling
    LDA VAR.PPUSCROLLX
    STA $2005
	LDA VAR.PPUSCROLLY
    STA $2005
LBL.NMI.RenderingEnabled.Exit:
RTS
